@@ -21,28 +21,52 @@ More information to follow...
 use 5.008005;
 use strict;
 use warnings;
-use Getopt::Long         2.37 ();
-use File::Remove         1.42 ();
-use Params::Util         0.35 ();
-use Time::HiRes        1.9709 ();
-use Time::Elapsed        0.24 ();
-use DBI                  1.57 ();
-use DBD::SQLite          1.25 ();
-use IO::Compress::Gzip  2.008 ();
-use IO::Compress::Bzip2 2.008 ();
-use DBIx::Publish             ();
+use File::Which            0.05 ();
+use File::Remove           1.42 ();
+use Getopt::Long           2.37 ();
+use Params::Util           0.35 ();
+use IPC::Run3             0.042 ();
+use Time::HiRes          1.9709 ();
+use Time::Elapsed          0.24 ();
+use DBI                    1.57 ();
+use DBD::SQLite            1.25 ();
+use IO::Compress::Gzip    2.008 ();
+use IO::Compress::Bzip2   2.008 ();
+use DBIx::Publish               ();
 
-our $VERSION = '0.07';
+our $VERSION = '0.08';
 
-use Object::Tiny 1.06 qw{
-	from
-	user
-	pass
-	to
-	index
-	sqlite_cache
-	argv
-};
+use constant MSWin32 => !! ( $^O eq 'MSWin32' );
+
+# Do we support lzma compression?
+my $CAN_LZMA = 0;
+if ( MSWin32 ) {
+	require Alien::Win32::LZMA;
+	$CAN_LZMA = 1;
+} elsif ( File::Which::which('lzma') ) {
+	$CAN_LZMA = 1;
+}
+
+use Moose 0.73;
+use MooseX::Types::Common::Numeric 0.001 'PositiveInt';
+
+has from         => ( is => 'ro', isa => 'Str' );
+has user         => ( is => 'ro', isa => 'Str' );
+has pass         => ( is => 'ro', isa => 'Str' );
+has to           => ( is => 'ro', isa => 'Str' );
+has index        => ( is => 'ro', isa => 'Int' );
+has trace        => ( is => 'ro', isa => 'Int' );
+has sqlite_cache => ( is => 'ro', isa => PositiveInt );
+has argv         => ( is => 'ro', isa => 'ArrayRef[Str]' );
+
+no Moose;
+
+
+
+
+
+#####################################################################
+# Constructor and Main Function
 
 sub main {
 	# Parse the command line options
@@ -63,15 +87,21 @@ sub main {
 		"sqlite_cache=i" => \$CACHE,
 	) or die("Failed to parse options");
 
+	# Prepend DBI: to the --from as a convenience if needed
+	if ( defined $FROM and $FROM !~ /^DBI:/ ) {
+		$FROM = "DBI:$FROM";
+	}
+
 	# Create the program instance
 	my $self = Xtract->new(
 		from         => $FROM,
 		user         => $USER,
 		pass         => $PASS,
 		to           => $TO,
-		index        => $INDEX,
+		index        => $INDEX ? 1 : 0,
 		trace        => $QUIET ? 0 : 1,
-		sqlite_cache => $CACHE,
+		$CACHE ? ( sqlite_cache => $CACHE ) : (),
+#		sqlite_cache => $CACHE,
 		argv         => [ @ARGV ],
 	);
 
@@ -79,17 +109,6 @@ sub main {
 	$self->run;
 }
 
-sub new {
-	my $self = shift->SUPER::new(@_);
-
-	# "DBI:" in --from is optional
-	unless ( $self->from =~ /^DBI:/ ) {
-		$self->{from} = "DBI:$self->{from}";
-	}
-
-	return $self;
-}
-
 sub to_gz {
 	$_[0]->to . '.gz';
 }
@@ -98,6 +117,14 @@ sub to_bz2 {
 	$_[0]->to . '.bz2';
 }
 
+sub to_lz {
+	if ( $CAN_LZMA ) {
+		return $_[0]->to . '.lz';
+	} else {
+		return;
+	}
+}
+
 
 
 
@@ -110,22 +137,28 @@ sub run {
 	my $start = Time::HiRes::time();
 
 	# Clear any existing output files
-	foreach my $file ( $self->to, $self->to_gz, $self->to_bz2 ) {
+	my @files = (
+		$self->to,
+		$self->to_gz,
+		$self->to_bz2,
+		$self->to_lz,
+	);
+	foreach my $file ( @files ) {
 		if ( defined $file and -e $file ) {
-			$self->trace("Deleting previous $file");
+			$self->say("Deleting $file");
 			File::Remove::remove($file);
 		}
 	}
 
 	# Connect to the data source
-	$self->trace("Connecting to data source " . $self->from);
+	$self->say("Connecting to data source " . $self->from);
 	my $source = DBI->connect( $self->from, $self->user, $self->pass, {
 		PrintError => 1,
 		RaiseError => 1,
 	} ) or die("Failed to connect to " . $self->from);
 
 	# Create the publish object
-	$self->trace("Creating SQLite database " . $self->to);
+	$self->say("Creating SQLite database " . $self->to);
 	my $publish = DBIx::Publish->new(
 		source       => $source,
 		file         => $self->to,
@@ -139,61 +172,83 @@ sub run {
 	}
 
 	# Get the list of tables
-	$self->trace("Configuring SQLite database");
+	$self->say("Configuring SQLite database");
 	$publish->prepare;
 	my @tables = grep { s/\"//g; $_ !~ /^sqlite_/ } $publish->source->tables;
 	foreach my $table ( @tables ) {
-		$self->trace("Publishing table $table");
+		$self->say("Publishing table $table");
 		my $tstart = Time::HiRes::time();
 		my $rows   = $publish->table( $table );
 		my $rate   = int($rows / (Time::HiRes::time() - $tstart));
-		$self->trace("Completed  table $table ($rows rows @ $rate/sec)");
+		$self->say("Completed  table $table ($rows rows @ $rate/sec)");
 	}
 	if ( $self->index ) {
 		foreach my $table ( @tables ) {
-			$self->trace("Indexing table $table");
+			$self->say("Indexing table $table");
 			$publish->index_table( $table );
 		}
 	}
-	$self->trace("Cleaning SQLite database");
+	$self->say("Cleaning SQLite database");
 	$publish->finish;
 
+	# Disconnect from the target database
+	$publish->dbh->disconnect;
+
 	# Compress the file
 	if ( $self->to_gz ) {
-		$self->trace("Creating gzip archive");
+		$self->say("Creating gzip archive");
 		IO::Compress::Gzip::gzip( $self->to => $self->to_gz )
 			or die 'Failed to gzip SQLite file';
 	}
 	if ( $self->to_bz2 ) {
-		$self->trace("Creating bzip2 archive");
+		$self->say("Creating bzip2 archive");
 		IO::Compress::Bzip2::bzip2( $self->to => $self->to_bz2 )
 			or die 'Failed to bzip2 SQLite file';
 	}
+	if ( $self->to_lz ) {
+		$self->say("Creating lzma archive");
+		if ( MSWin32 ) {
+			Alien::Win32::LZMA::lzma_compress(
+				$self->to => $self->to_lz,
+			) or die 'Failed to lzma SQLite file';
+		} else {
+			my $lzma   = File::Which::which('lzma');
+			my $stdout = '';
+			my $stderr = '';
+			IPC::Run3::run3(
+				[ $lzma, 'e', $self->to, $self->to_lz ],
+				\undef, \$stdout, \$stderr,
+			) or die 'Failed to lzma SQLite file';
+		}
+	}
 
 	# Clean up
 	$source->disconnect;
-	$self->trace(
+	$self->say(
 		"Extraction completed in " .
 		Time::Elapsed::elapsed(int(Time::HiRes::time() - $start)) .
 		" seconds"
 	);
 
 	# Summarise the run
-	$self->trace("Created " . $self->to);
+	$self->say("Created " . $self->to);
 	if ( $self->to_gz ) {
-		$self->trace("Created " . $self->to_gz);
+		$self->say("Created " . $self->to_gz);
 	}
 	if ( $self->to_bz2 ) {
-		$self->trace("Created " . $self->to_bz2);
+		$self->say("Created " . $self->to_bz2);
+	}
+	if ( $self->to_lz ) {
+		$self->say("Created " . $self->to_lz);
 	}
 
 	return 1;
 }
 
-sub trace {
-	if ( Params::Util::_CODE($_[0]->{trace}) ) {
-		$_[0]->trace( @_[1..$#_] );
-	} elsif ( $_[0]->{trace} ) {
+sub say {
+	if ( Params::Util::_CODE($_[0]->trace) ) {
+		$_[0]->say( @_[1..$#_] );
+	} elsif ( $_[0]->trace ) {
 		my $t = scalar localtime time;
 		print map { "[$t] $_\n" } @_[1..$#_];
 	}
